<?php

class DbCollectionItem extends DbEntity
{
	/* Maps a database reader object to a business object
	 * @obj: database reader object
	 * returns: a business object */
	protected function map($obj)
	{
		$item = new CollectionItem();
		$item->id = $obj->id;
		$item->title = $obj->title;
		$item->url_title = $obj->url_title;
		$item->description = $obj->description;
		$item->date_created = $obj->date_created;
		$item->collection_id = $obj->collection_id;
		$item->image = $obj->image;
		$item->comments_closed = $obj->comments_closed;
		$item->visible = $obj->visible;
		$item->meta_keywords = $obj->meta_keywords;
		$item->meta_description = $obj->meta_description;
		return $item;
	}
	
	/* Queries the database for a collectionitem with a specific id
	 * @itemid: the id of the collectionitem
	 * returns: a collectionitem or null */
	public function GetById($itemid)
	{
		$item = null;
		if ($result = $this->mysqli->query(
			'select ci.*, cic.content1, cic.content2, group_concat(distinct ct.tagname order by ct.tagname) as tags,
			count(distinct cicom.id) as accepted_comments_count
			from (collectionitems as ci, collectionitems_content as cic)
			left join collectionitems_tags as cit on cit.item_id = ci.id
			left join collectiontags as ct on ct.id = cit.tag_id
			left join collectionitems_comments as cicom on (cicom.item_id = ci.id and cicom.accepted = 1)
			where ci.id = ' . $this->escape($itemid) . ' and ci.id = cic.item_id group by ci.id'))
		{
			if ($obj = $result->fetch_object())
			{
				$item = $this->map($obj);
				$item->content1 = $obj->content1;
				$item->content2 = $obj->content2;
				$item->tags = empty($obj->tags) ? null : explode(',', $obj->tags);
				$item->accepted_comments_count = $obj->accepted_comments_count;
			}
			$result->close();
		}
		return $item;
	}
	
	/* Queries the database for all collectionitems within a specific
	 * collection that are tagged with a specific tagname limited by offset and count
	 * @collection: the collection the items must be member of
	 * @tagname: the tagname the items must be tagged with
	 * @offset: the start of the resultset
	 * @count: the maximum allowed items in the resultset
	 * returns: an array of collectionitems ordered by the creationdate or an empty array */
	public function GetItemsByTagname(Collection $collection, $tagname, $offset, $count)
	{
		$items = array();
		if ($result = $this->mysqli->query(
			'select ci.*, cic.content1, cic.content2 <> \'\' as has_content2, group_concat(distinct ct.tagname order by ct.tagname) as tags,
			count(distinct cicom.id) as accepted_comments_count
			from (collectionitems as ci, collectionitems_content as cic)
			left join collectionitems_tags as cit on cit.item_id = ci.id
			left join collectiontags as ct on ct.id = cit.tag_id
			left join collectionitems_comments as cicom on (cicom.item_id = ci.id and cicom.accepted = 1)
			where ci.collection_id = ' . $collection->id . ' and ci.visible = 1 and ci.id = cic.item_id
			and (select count(*) from collectiontags as ct, collectionitems_tags as cit where
			cit.item_id = ci.id and cit.tag_id = ct.id
			and ct.tagname = \'' . $tagname . '\') = 1
			group by ci.id
			order by ci.date_created desc limit ' . $offset . ', ' . $count))
		{
			while ($obj = $result->fetch_object())
			{
				$item = $this->map($obj);
				$item->content1 = $obj->content1;
				$item->has_content2 = $obj->has_content2;
				$item->tags = empty($obj->tags) ? null : explode(',', $obj->tags);
				$item->accepted_comments_count = $obj->accepted_comments_count;
				$items[] = $item;
			}
			$result->close();
		}
		return $items;
	}
	
	/* Queries the database for collectionitems in a specific collection,
	 * optionally filtered by year, month and day and limited by offset and count
	 * @collection: the collection the items must be member of
	 * @year: a year (4 digits) or null
	 * @month: a month (2 digits) or null
	 * @day: a day (2 digits) or null
	 * @offset: the start of the resultset
	 * @count: the maximum allowed items in the resultset
	 * returns: an array of collectionitems ordered by the creationdate or an empty array */
	public function GetItems(Collection $collection, $year, $month, $day, $offset, $count)
	{
		$query = array();
		if (!empty($year)) $query[] = 'year(ci.date_created) = ' . $year;
		if (!empty($month)) $query[] = 'month(ci.date_created) = ' . $month;
		if (!empty($day)) $query[] = 'day(ci.date_created) = ' . $day;
		if (count($query) > 0)
			$query = 'and ' . implode(' and ', $query);
		else
			$query = '';
		
		$items = array();
		if ($result = $this->mysqli->query(
			'select ci.*, cic.content1, cic.content2 <> \'\' as has_content2, group_concat(distinct ct.tagname order by ct.tagname) as tags,
			count(distinct cicom.id) as accepted_comments_count
			from (collectionitems as ci, collectionitems_content as cic)
			left join collectionitems_tags as cit on cit.item_id = ci.id
			left join collectiontags as ct on ct.id = cit.tag_id
			left join collectionitems_comments as cicom on (cicom.item_id = ci.id and cicom.accepted = 1)
			where ci.collection_id = ' . $collection->id . ' and ci.visible = 1 and ci.id = cic.item_id ' . $query . '
			group by ci.id
			order by ci.date_created desc limit ' . $offset . ', ' . $count))
		{
			while ($obj = $result->fetch_object())
			{
				$item = $this->map($obj);
				$item->content1 = $obj->content1;
				$item->has_content2 = $obj->has_content2;
				$item->tags = empty($obj->tags) ? null : explode(',', $obj->tags);
				$item->accepted_comments_count = $obj->accepted_comments_count;
				$items[] = $item;
			}
			$result->close();
		}
		return $items;
	}
	
	/* Queries the database for one specific collectionitem with the unique combination of year, month, day and title
	 * @collection: the collection the items must be member of
	 * @year: a year (4 digits)
	 * @month: a month (2 digits)
	 * @day: a day (2 digits)
	 * @title: the title of the item
	 * returns: a collectionitem or null */
	public function GetItem(Collection $collection, $year, $month, $day, $url_title)
	{
		$item = null;
		if ($result = $this->mysqli->query(
			'select ci.*, cic.content1, cic.content2, group_concat(distinct ct.tagname order by ct.tagname) as tags
			from (collectionitems as ci, collectionitems_content as cic)
			left join collectionitems_tags as cit on cit.item_id = ci.id
			left join collectiontags as ct on ct.id = cit.tag_id
			where ci.collection_id = ' . $collection->id . '
			and ci.id = cic.item_id and year(ci.date_created) = ' . $year . '
			and month(ci.date_created) = ' . $month . '
			and day(ci.date_created) = ' . $day . '
			and ci.url_title = \'' . $url_title . '\'
			group by ci.id'))
		{
			if ($obj = $result->fetch_object())
			{
				$item = $this->map($obj);
				$item->content1 = $obj->content1;
				$item->content2 = $obj->content2;
				$item->tags = empty($obj->tags) ? null : explode(',', $obj->tags);
			}
			$result->close();
		}
		return $item;
	}
}

class DbCollection extends DbEntity
{
	/* Maps a database reader object to a business object
	 * @obj: the database reader object
	 * returns: the business object */
	protected function map($obj)
	{
		$s = new Collection();
		$s->id = $obj->id;
		$s->name = $obj->name;
		$s->has_description = $obj->has_description;
		$s->has_image = $obj->has_image;
		$s->has_content = $obj->has_content;
		$s->has_comments = $obj->has_comments;
		$s->comments_accepted_default = $obj->comments_accepted_default;
		$s->comments_add_email = $obj->comments_add_email;
		return $s;
	}
	
	/* Queries the database for a specific collection
	 * @id: the collectionid
	 * @tagname: when not null, it limits the items_count property with only the items that are tagged by this tag
	 * returns: a collection or null */
	public function GetById($id, $tagname)
	{
		$query = empty($tagname) ?
			'select c.*, count(distinct ci.id) as items_count, count(distinct ci1.id) as items_visible_count,
			count(distinct cic.id) as not_accepted_comments_count
			from (collections as c)
			left join collectionitems as ci on (c.id = ci.collection_id)
			left join collectionitems_comments as cic on (cic.item_id = ci.id and cic.accepted = 0)
			left join collectionitems as ci1 on (c.id = ci1.collection_id and ci1.visible = 1)
			where c.id = ' . $id . ' group by c.id'
			:
			'select c.*, count(distinct ci.id) as items_count, count(distinct ci1.id) as items_visible_count,
			count(distinct cic.id) as not_accepted_comments_count
			from (collections as c, collectiontags as ct, collectionitems_tags as cit)
			left join collectionitems as ci on (c.id = ci.collection_id and ci.id = cit.item_id)
			left join collectionitems_comments as cic on (cic.item_id = ci.id and cic.accepted = 0)
			left join collectionitems as ci1 on (c.id = ci1.collection_id and ci1.id = cit.item_id and ci1.visible = 1)
			where c.id = ' . $id . ' and ct.collection_id = c.id and ct.tagname = \'' . $this->escape($tagname) . '\' and ct.id = cit.tag_id group by c.id';
		$c = null;
		if ($result = $this->mysqli->query($query))
		{
			if ($obj = $result->fetch_object())
			{
				$c = $this->map($obj);
				$c->items_count = $obj->items_count;
				$c->items_visible_count = $obj->items_visible_count;
				$c->not_accepted_comments_count = $obj->not_accepted_comments_count;
			}
			$result->close();
		}
		return $c;
	}
	
	/* Queries the database for how many times a tag is used for the visible items in this collection
	 * @collectionid: the collectionid
	 * returns: an associative array with key = tagname, value = number of occurrences or an empty array */
	public function GetTagCount($collectionid)
	{
		$tags = array();
		if ($result = $this->mysqli->query(
			'select ct.tagname, count(*) as total
			from (collectiontags as ct, collectionitems_tags as cit, collectionitems as ci)
			where ct.collection_id = ' . $collectionid . ' and ct.id = cit.tag_id
			and cit.item_id = ci.id and ci.visible = 1
			group by ct.tagname'))
		{
			while ($obj = $result->fetch_object())
			{
				$tags[$obj->tagname] = $obj->total;
			}
			$result->close();
		}
		return $tags;
	}
}

class DbComment extends DbEntity
{
	/* Maps a database reader object to a Comment business object
	 * @obj: database reader
	 * returns: a Comment object */
	protected function map($obj)
	{
		$c = new Comment();
		$c->id = $obj->id;
		$c->itemId = $obj->item_id;
		$c->dateCreated = $obj->date_created;
		$c->name = $obj->name;
		$c->comment = $obj->comment;
		$c->accepted = $obj->accepted;
		$c->email = $obj->email;
		$c->url = $obj->url;
		$c->specification = $obj->specification;
		return $c;
	}
	
	/* Gets the accepted comments in a specific collection
	 * @collection_id: id of collection
	 * returns: array of comments */
	public function GetRecentAcceptedFromCollection($collection_id)
	{
		$comments = array();
		if ($result = $this->mysqli->query(
			'select cic.* from collectionitems_comments as cic, collectionitems as ci' .
				' where ci.collection_id = ' . $collection_id . ' and ci.id = cic.item_id' . 
				' and cic.accepted = 1 order by date_created desc limit 10'))
		{
			while ($obj = $result->fetch_object())
			{
				$comments[] = $this->map($obj);
			}
			$result->close();
		}
		return $comments;
	}
	
	/* Queries the database for comments that are accepted within a specific collectionitem
	 * @item_id: the id of the collectionitem
	 * returns: an array with comments or an empty array */
	public function GetAccepted($item_id)
	{
		$comments = array();
		if ($result = $this->mysqli->query(
			'select * from collectionitems_comments where item_id = ' . $item_id . ' and accepted = 1 order by date_created'))
		{
			while ($obj = $result->fetch_object())
			{
				$comments[] = $this->map($obj);
			}
			$result->close();
		}
		return $comments;
	}

	/* Add a comment to the database
	 * @comment: a Comment object
	 * returns: true on success, false on failure */
	public function add(Comment $comment)
	{
		return $this->mysqli->query(
			'insert into collectionitems_comments set item_id = ' . $this->escape($comment->itemId) . '
			, name = \'' . $this->escape($comment->name) . '\', email = \'' . $this->escape($comment->email) . '\'
			, comment = \'' . $this->escape($comment->comment) . '\', url = \'' . $this->escape($comment->url) . '\'
			, accepted = ' . $this->escape($comment->accepted) . ', specification = \'' . $comment->specification . '\'');
	}
}

?>